﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NHibernate.Dialect;
using NUnit.Framework;
using NHibernate.Linq;

namespace NHibernate.Test.NHSpecificTest.NH2789
{
    [TestFixture]
    public class SampleTest : BugTestCase
    {
        protected override void OnSetUp()
        {
            base.OnSetUp();
            using (ISession session = this.OpenSession())
            {
                DomainClass entity = new DomainClass();
                entity.Id = 1;
                entity.NullableByteProperty = 16;
                entity.NullableIntProperty = 1024;
                session.Save(entity);
                session.Flush();
            }
        }

        protected override void OnTearDown()
        {
            base.OnTearDown();
            using (ISession session = this.OpenSession())
            {
                string hql = "from System.Object";
                session.Delete(hql);
                session.Flush();
            }
        }

        protected override bool AppliesTo(NHibernate.Dialect.Dialect dialect)
        {
            return dialect as MsSql2005Dialect != null;
        }

        [Test]
        public void LinqQueryOnNullableByteShouldWork()
        {
            using (ISession session = this.OpenSession())
            {
                // DomainClass entity = session.Get<DomainClass>(1);

                // try LINQ join on byte property
                var entityQueryable = from x in session.Query<DomainClass>()
                                      where x.NullableByteProperty == Convert.ToByte(16)
                                      select x;

                Assert.IsTrue(entityQueryable.Count() == 1);
            }
        }

        [Test]
        public void LinqQueryOnNullableIntShouldWork()
        {
            using (ISession session = this.OpenSession())
            {
                // DomainClass entity = session.Get<DomainClass>(1);

                // try LINQ join on byte property
                var entityQueryable = from x in session.Query<DomainClass>()
                                      where x.NullableIntProperty == Convert.ToByte(2)
                                      select x;

                // fails
                var personList = entityQueryable.ToList();

                Assert.IsTrue(entityQueryable.Count() == 0);
            }
        }

    }
}
